package ru.dbotthepony.mc.otm.datagen.recipes

import net.minecraft.data.recipes.RecipeCategory
import net.minecraft.data.recipes.RecipeOutput
import net.minecraft.data.recipes.ShapelessRecipeBuilder
import net.minecraft.tags.TagKey
import net.minecraft.world.item.DyeColor
import net.minecraft.world.item.Item
import net.minecraft.world.item.ItemStack
import net.minecraft.world.item.Items
import net.minecraft.world.item.crafting.Ingredient
import net.minecraft.world.level.ItemLike
import net.neoforged.neoforge.common.Tags
import ru.dbotthepony.mc.otm.util.registryName
import ru.dbotthepony.mc.otm.datagen.modLocation
import ru.dbotthepony.mc.otm.registry.MItemTags
import ru.dbotthepony.mc.otm.registry.game.MItems
import ru.dbotthepony.mc.otm.registry.MRegistry

fun hammerRecipe(output: ItemLike, input: ItemLike, consumer: RecipeOutput) {
	ShapelessRecipeBuilder(RecipeCategory.MISC, output, 1)
		.requires(MItemTags.TOOLS_HAMMERS)
		.requires(input)
		.unlockedBy(MItemTags.TOOLS_HAMMERS)
		.unlockedBy(input)
		.save(consumer)
}

fun hammerRecipe(output: ItemLike, input: TagKey<Item>, consumer: RecipeOutput) {
	ShapelessRecipeBuilder(RecipeCategory.MISC, output, 1)
		.requires(MItemTags.TOOLS_HAMMERS)
		.requires(input)
		.unlockedBy(MItemTags.TOOLS_HAMMERS)
		.unlockedBy(input)
		.save(consumer)
}

fun addShapelessRecipes(consumer: RecipeOutput) {
	for (color in DyeColor.entries) {
		ShapelessRecipeBuilder(RecipeCategory.TRANSPORTATION, MItems.CARGO_CRATE_MINECARTS[color]!!, 1)
			.requires(Items.MINECART)
			.requires(MRegistry.CARGO_CRATES.items[color]!!)
			.unlockedBy(Items.MINECART)
			.unlockedBy(MRegistry.CARGO_CRATES.items[color]!!)
			.save(consumer)

		ShapelessRecipeBuilder(RecipeCategory.TRANSPORTATION, MItems.CARGO_CRATE_MINECARTS[color]!!, 1)
			.requires(Ingredient.of(MItems.CARGO_CRATE_MINECARTS.entries.stream().filter { it.key != color }.map { ItemStack(it.value) }))
			.requires(color.tag)
			.unlockedBy(MItems.CARGO_CRATE_MINECARTS.entries.stream().filter { it.key != color }.map { it.value })
			.save(consumer, modLocation(MItems.CARGO_CRATE_MINECARTS[color]!!.registryName!!.path + "_alt"))
	}

	ShapelessRecipeBuilder(RecipeCategory.TRANSPORTATION, MItems.CARGO_CRATE_MINECARTS[null]!!, 1)
		.requires(Items.MINECART)
		.requires(MRegistry.CARGO_CRATES.item)
		.unlockedBy(Items.MINECART)
		.unlockedBy(MRegistry.CARGO_CRATES.item)
		.save(consumer)

	ShapelessRecipeBuilder(RecipeCategory.BUILDING_BLOCKS, MItems.TRITANIUM_INGOT_BLOCK, 1)
		.requires(Ingredient.of(MItemTags.TRITANIUM_INGOTS), 9)
		.unlockedBy(MItemTags.TRITANIUM_INGOTS)
		.save(consumer)

	ShapelessRecipeBuilder(RecipeCategory.BUILDING_BLOCKS, MItems.TRITANIUM_RAW_BLOCK, 1)
		.requires(Ingredient.of(MItemTags.TRITANIUM_ORE_CLUMPS), 9)
		.unlockedBy(MItemTags.TRITANIUM_ORE_CLUMPS)
		.save(consumer)

	ShapelessRecipeBuilder(RecipeCategory.MISC, MItems.TRITANIUM_ORE_CLUMP, 9)
		.requires(Ingredient.of(MItemTags.RAW_TRITANIUM_STORAGE))
		.unlockedBy(MItemTags.RAW_TRITANIUM_STORAGE)
		.save(consumer, modLocation("raw_tritanium_from_storage"))

	ShapelessRecipeBuilder(RecipeCategory.MISC, MItems.TRITANIUM_INGOT, 9)
		.requires(Ingredient.of(MItemTags.TRITANIUM_INGOTS_STORAGE))
		.unlockedBy(MItemTags.TRITANIUM_INGOTS_STORAGE)
		.save(consumer, modLocation("tritanium_ingot_from_storage"))

	ShapelessRecipeBuilder(RecipeCategory.MISC, MItems.TRITANIUM_NUGGET, 9)
		.requires(MItemTags.TRITANIUM_INGOTS)
		.unlockedBy(MItemTags.TRITANIUM_INGOTS)
		.unlockedBy(MItemTags.TRITANIUM_NUGGETS)
		.save(consumer)

	ShapelessRecipeBuilder(RecipeCategory.MISC, MItems.TRITANIUM_INGOT, 1)
		.requires(Ingredient.of(MItemTags.TRITANIUM_NUGGETS), 9)
		.unlockedBy(MItemTags.TRITANIUM_INGOTS)
		.unlockedBy(MItemTags.TRITANIUM_NUGGETS)
		.save(consumer, modLocation("ingot_from_nuggets"))

	hammerRecipe(MItems.TRITANIUM_PLATE, MItemTags.TRITANIUM_INGOTS, consumer)
	hammerRecipe(MItems.IRON_PLATE, Tags.Items.INGOTS_IRON, consumer)
	hammerRecipe(MItems.GOLD_PLATE, Tags.Items.INGOTS_GOLD, consumer)

	ShapelessRecipeBuilder(RecipeCategory.BUILDING_BLOCKS, MItems.WITHERED_STEEL_BLOCK, 1)
		.requires(Ingredient.of(MItems.WITHERED_STEEL), 9)
		.unlockedBy(MItems.WITHERED_STEEL)
		.save(consumer)

	ShapelessRecipeBuilder(RecipeCategory.MISC, MItems.WITHERED_STEEL, 9)
		.requires(Ingredient.of(MItems.WITHERED_STEEL_BLOCK))
		.unlockedBy(MItems.WITHERED_STEEL_BLOCK)
		.save(consumer)

	ShapelessRecipeBuilder(RecipeCategory.BUILDING_BLOCKS, MItems.ROFLITE_ALLOY_BLOCK, 1)
		.requires(Ingredient.of(MItems.ROFLITE_ALLOY_INGOT), 9)
		.unlockedBy(MItems.ROFLITE_ALLOY_INGOT)
		.save(consumer)

	ShapelessRecipeBuilder(RecipeCategory.MISC, MItems.ROFLITE_ALLOY_INGOT, 9)
		.requires(Ingredient.of(MItems.ROFLITE_ALLOY_BLOCK))
		.unlockedBy(MItems.ROFLITE_ALLOY_BLOCK)
		.save(consumer)
}
